use v6;
use Test;

use Facter;
use Facter::Util::Fact;
use Facter::Util::Collection;
use Facter::Util::Loader;
use Facter::Util::Resolution;
use Facter::Util::Confine;
use Facter::Util::Values;

ok(1, 'Facter main classes loaded');

done;

